each name = 'SameGame-Game''From etoys2.2 of 1 October 2007 [latest update: #1675] on 7 December 2007 at 2:48:30 pm'!
"Change Set:		SameGame-Game
Date:			7 December 2007
Author:			Robert Krahn

Authors: Martin Boissier, Marco Hornung, Eyk Kny, Tobias Rawald, Keven Richly, Jan-Felix Schwarz, and Viktor Stoitschev

Hasso-Plattner-Institut
Software Architecture Group
http://www.hpi.uni-potsdam.de/swa/"!

BorderedMorph subclass: #SameGame
	instanceVariableNames: 'board scoreDisplay selectionDisplay helpText stoneDisplays controlMenuMorph scoreDisplayMorph fieldMorph '
	classVariableNames: 'Colors Images '
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGame commentStamp: '<historical>' prior: 0!
Class: SameGame
	A class representing the game form. 
Description:
	It provides methods for creating the gui. It handels gui interaction such as MouseDowns on buttons and updates the displays for game statictics.
Collaboration:
	SameGameGameField, SameGameStoneDisplay
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!

SameGame class
	instanceVariableNames: 'description '!
RectangleMorph subclass: #SameGameGameField
	instanceVariableNames: 'game stoneCatalog score numberOfStones maxStoneColumns maxStoneRows maxFieldSizeLimit minFieldSizeLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGameGameField commentStamp: 'jfs 7/12/2007 21:35' prior: 0!
Class: SameGameGameField
	A subclass of RectangleMorph representing the game field.
Description:
	Provides methods for filling the game field with stones, adjusting field size, and finding stone groups. Game score is stored in instance variable. Takes instances of SameGameStoneColumn as submorphs.
Collaboration:
	SameGame. SameGameStone, SameGameStoneColumn. SameGameStoneGroup
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!

ImageMorph subclass: #SameGameStone
	instanceVariableNames: 'gameField stoneGroup stoneImage stoneImageHover stoneColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGameStone commentStamp: 'jfs 7/12/2007 20:41' prior: 0!
Class: SameGameStone
	A subclass of ImageMorph representing a stone on the game field.
Description:
	Provides functionality for gui interaction (handles mouse events and offers methods for visual effects). The instance variable stoneColor defines the type of the stone.
Collaboration:
	SameGameGameField. SameGameStoneGroup
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!

Morph subclass: #SameGameStoneColumn
	instanceVariableNames: 'maxStones'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGameStoneColumn commentStamp: 'jfs 7/12/2007 20:03' prior: 0!
Class: SameGameStoneColumn
	A subclass of Morph representing a stack of stones to be placed on the game field.
Description:
	This class takes instances of SameGameStone as submorphs. It provides methods to access a stone at a certain row. Realizes the falling down of stones by using the layout properties of the Morph superclass.
Collaboration:
	SameGameStone
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!

RectangleMorph subclass: #SameGameStoneDisplay
	instanceVariableNames: 'value stoneColor gameField string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGameStoneDisplay commentStamp: 'jfs 7/19/2007 11:13' prior: 0!
Class: SameGameStoneDisplay
	A subclass of RectangleMorph representing the display of a number of stones.
Description:
	Can take a value (number of stones of a certain color) and displays it in connection with the stone morph.
Collaboration:
	SameGameGameField
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!

Set subclass: #SameGameStoneGroup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SameGame-Game'!

!SameGameStoneGroup commentStamp: 'jfs 7/12/2007 19:59' prior: 0!
Class: SameGameStoneGroup
	A class representing a group of neighboring stones of same color.
Description:
	This class extends the functionality of a Set by methods for visual effects and  evaluating score.
Collaboration:
	SameGameStone
Author:
	HPI SWA2007 Team19
Date:
	07/12/2007!


!Object methodsFor: '*SameGame-Game' stamp: 'ek 6/5/2007 07:12'!
isSameGameStone
	^ false! !


!SameGame methodsFor: 'initialization' stamp: 'rkrk 11/16/2007 18:16'!
initialize

	super initialize.
	self class addImages.

	fieldMorph := SameGameGameField new.
	
	self
		makeLayout;
		makeBackground;
		makeScoreDisplay;
		makeLowerMenu;
		makeField;
		makeUpperMenu;
		updateDisplays! !

!SameGame methodsFor: 'initialization' stamp: 'jfs 7/17/2007 12:19'!
makeField
	"Place gui component for the game field on the game form."

	fieldMorph game: self.
	self addMorph: fieldMorph! !

!SameGame methodsFor: 'initialization' stamp: 'ek 6/30/2007 17:14'!
makeLayout

	self
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: 4@10;
		cellInset: 4;
		extent: SameGame defaultExtent;
		center: World center;
		useRoundedCorners;
		borderWidth: 2! !

!SameGame methodsFor: 'initialization' stamp: 'ek 7/17/2007 15:58'!
makeLowerMenu
	"Place gui components for resizing field size and displaying numbers of stone for each color on the game form."

	|  anOwnerForLowerMenu |
	stoneDisplays := OrderedCollection new.
	
	anOwnerForLowerMenu := self makeOwnerForLowerMenu.
	
	fieldMorph stoneCatalog keysDo: [ :aColor | 
						stoneDisplays add: (SameGameStoneDisplay newWithStoneColor: aColor at: fieldMorph)].
	
	anOwnerForLowerMenu addMorph: 
						(self makeButtonWithImage: (SameGame image: #buttonPlus) action: #increaseFieldSize ballonText: 'increase field size').
	
	stoneDisplays do: [:aStoneMenu | 
						anOwnerForLowerMenu addMorph: aStoneMenu].
	
	anOwnerForLowerMenu addMorph: 
						(self makeButtonWithImage: (SameGame image: #buttonMinus) action: #decreaseFieldSize ballonText: 'decrease field size').
	
	self addMorph: anOwnerForLowerMenu
! !

!SameGame methodsFor: 'initialization' stamp: 'jfs 7/17/2007 12:17'!
makeScoreDisplay
	"Place gui component for displaying score on the gema form."

	self addMorph: 
		(scoreDisplayMorph := StringMorph new
			contents: 0 asString;
			fontName: 'ComicSansMS' size: 35;
			color:  (SameGame color: #darkBlue);
			yourself)
		! !

!SameGame methodsFor: 'initialization' stamp: 'jfs 7/17/2007 12:24'!
makeUpperMenu
	"Place gui components for starting a new game and closing the game on the game form. Display headline."

	self addMorph:
		(controlMenuMorph := self makeOwnerForUpperMenu
			addMorph: (self makeButtonWithImage: (SameGame image: #buttonClose) action: #closeGame ballonText: 'close game');
			addMorph: self makeHeadline;
			addMorph: (self makeButtonWithImage: (SameGame image: #buttonNew) action: #newGame ballonText: 'start new game');
			yourself)! !

!SameGame methodsFor: 'actions' stamp: 'ek 6/24/2007 15:41'!
closeGame

	self delete! !

!SameGame methodsFor: 'actions' stamp: 'jfs 7/17/2007 12:13'!
decreaseFieldSize
	"Remove two columns and two rows from the game field and start a new game."

	fieldMorph sizeDownBy: 2.
	self newGame! !

!SameGame methodsFor: 'actions' stamp: 'jfs 7/17/2007 12:13'!
increaseFieldSize
	"Add two more columns and two more rows to the game field and start a new game."

	fieldMorph sizeUpBy: 2.
	self newGame! !

!SameGame methodsFor: 'actions' stamp: 'ek 6/24/2007 16:25'!
makeBackground

	self 
		color: (SameGame color: #lightBlue);
		borderColor: (SameGame color: #darkBlue)! !

!SameGame methodsFor: 'actions' stamp: 'ek 6/30/2007 15:45'!
newGame

	fieldMorph reset.
	self updateDisplays.
	scoreDisplayMorph color: (SameGame color: #darkBlue)! !

!SameGame methodsFor: 'actions' stamp: 'ek 6/24/2007 15:42'!
scoreDisplayMorph

	^ scoreDisplayMorph! !

!SameGame methodsFor: 'actions' stamp: 'jfs 7/17/2007 12:11'!
updateDisplays
	"Update gui outputs for score and numbers of stones of each color."

	stoneDisplays do: [ :aDisplay | aDisplay value: ((fieldMorph numberOfStones) at: (aDisplay stoneColor)) ].

	scoreDisplayMorph contents: (fieldMorph score) asString.
	
	fieldMorph gameIsOver ifTrue: [ scoreDisplayMorph color: (SameGame color: #orange) ]! !

!SameGame methodsFor: 'submorphs' stamp: 'jfs 7/17/2007 12:23'!
makeButtonWithImage: anImage action: aSelector ballonText: aString

	^ IconicButton new
		vResizing: #shrinkWrap;
		hResizing: #shrinkWrap;
		target: self;
		actionSelector: aSelector;
		labelGraphic: anImage;
		setBalloonText: aString;
		borderWidth: 0;
		color: Color transparent;
		yourself! !

!SameGame methodsFor: 'submorphs' stamp: 'jfs 7/17/2007 12:23'!
makeHeadline

	^ ImageMorph new
		image: (SameGame image: #headline);
		yourself! !

!SameGame methodsFor: 'submorphs' stamp: 'jfs 7/17/2007 12:21'!
makeOwnerForLowerMenu

	^ RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		listCentering: #center;
		hResizing: #shrinkWrap;
	 	cellInset: 1;
		height: 30;
		borderWidth: 0;
		color: Color transparent;
		yourself! !

!SameGame methodsFor: 'submorphs' stamp: 'jfs 7/17/2007 12:21'!
makeOwnerForUpperMenu

	^ RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		listCentering: #center;
		hResizing: #shrinkWrap;
		cellInset: 13;
		height: 50;
		borderWidth: 0;
		color: Color transparent;
		yourself! !


!SameGame class methodsFor: 'parts bin' stamp: 'ek 6/24/2007 11:22'!
descriptionForPartsBin

	^ self
		partName: 'SameGame'
		categories: #('Games')
		documentation: 'The game was created in the eighties by Kuniaki Moribe.'! !

!SameGame class methodsFor: 'default settings' stamp: 'rkrk 11/28/2007 13:34'!
addImages
	| stones |
	Images := Dictionary new.
	stones := #(#greenStone #redStone #yellowStone #blueStone #purpleStone #orangeStone #buttonClose #buttonNew #headline #buttonPlus #buttonMinus).
	stones do: [:each | 
		Images
			at: each
			put: (self formFromFileNamed: each asString)].

	"Images ifNil: [ | cs |
		cs := ChangeSet allInstances detect: [:each | each name = 'SameGame-Game.cs'].
		Images := cs structures at: #Images]"! !

!SameGame class methodsFor: 'default settings' stamp: 'ek 6/24/2007 19:52'!
defaultDirectory

	^ 'SameGame', FileDirectory slash! !

!SameGame class methodsFor: 'default settings' stamp: 'ek 6/24/2007 15:34'!
defaultExtent

	^ 460@450! !

!SameGame class methodsFor: 'default settings' stamp: 'rkrk 11/16/2007 17:04'!
images

	^ Images! !

!SameGame class methodsFor: 'default settings' stamp: 'ek 6/24/2007 19:53'!
setColors
	
	Colors := Dictionary new
		at: #lightBlue put: (Color r: 0.835 g: 0.89 b: 0.925);
		at: #darkBlue put: (Color r: 0.58 g: 0.717 b: 0.725);
		at: #blue put: (Color r: 0.714 g: 0.807 b: 0.815);
		at: #orange put: (Color r: 1.0 g: 0.384 b: 0.176);
		yourself
	! !

!SameGame class methodsFor: 'instance creation' stamp: 'ek 7/1/2007 16:30'!
startNewGame

	self new openInWorld! !

!SameGame class methodsFor: 'accessing' stamp: 'ek 6/24/2007 19:53'!
color: aSymbol

	^ Colors at: aSymbol! !

!SameGame class methodsFor: 'accessing' stamp: 'ek 7/17/2007 10:35'!
description

	^ description! !

!SameGame class methodsFor: 'accessing' stamp: 'ek 6/24/2007 19:53'!
image: aSymbol

	^ Images at: aSymbol! !

!SameGame class methodsFor: 'installation' stamp: 'rkrk 11/16/2007 13:55'!
formFromFileNamed: aString

	^ ImageReadWriter formFromFileNamed: SameGame defaultDirectory , aString , '.png'.
	"^ Morph new imageForm"
	"^ self perform: aString asSymbol"! !

!SameGame class methodsFor: 'installation' stamp: 'ek 7/18/2007 12:44'!
install

	self addImages.

	description := 
		'This is the well-known puzzle game SameGame. The first version was developed in the eighties by Kuniaki Moribe.',(Character cr) asString,
		(Character cr) asString,
		'How to start the game:',(Character cr) asString,
		'- Open the objects tool -> select <<Games>> -> <<SameGame>> (drag it into the world)',(Character cr) asString,
		'- Or choose <<SameGame startNewGame>>.'! !

!SameGame class methodsFor: 'initialization' stamp: 'ek 7/18/2007 12:44'!
initialize

	self setColors! !


!SameGameGameField methodsFor: 'settings' stamp: 'ek 6/24/2007 15:43'!
fillStoneCatalog

	stoneCatalog := Dictionary new
		at: #green put: (SameGameStone greenStoneAt: self);
	 	at: #yellow put: (SameGameStone yellowStoneAt: self);
		at: #red put: (SameGameStone redStoneAt: self);
		at: #blue put: (SameGameStone blueStoneAt: self);
	 	at: #purple put: (SameGameStone purpleStoneAt: self);
		yourself! !

!SameGameGameField methodsFor: 'actions' stamp: 'jfs 7/19/2007 11:41'!
adaptExtend
	"Adapt the extend of the morph to the max number of stone rows and columns."

	| stoneSize fieldWidth fieldHeight |
	stoneSize := 25.
	fieldWidth := (stoneSize * maxStoneColumns) + (2 * borderWidth).
	fieldHeight := (stoneSize * maxStoneRows) + (2 * borderWidth).
	
	self extent: fieldWidth @ fieldHeight! !

!SameGameGameField methodsFor: 'actions' stamp: 'jfs 7/19/2007 11:48'!
countStones
	"Count the stones of every color and store the numbers in the numberOfStones dictionary."
	
	numberOfStones keys do: [ :aKey | numberOfStones at: aKey put: 0 ].
	
	self stoneColumns do: [ :aColumn | 
		aColumn stones do: [ :aStone | 
			numberOfStones at: (aStone stoneColor) put: (numberOfStones at: aStone stoneColor) + 1 ] ]
	
	! !

!SameGameGameField methodsFor: 'actions' stamp: 'ek 6/24/2007 12:27'!
destroyEmptyStoneColumns

	self stoneColumns do: [ :column | column destroyIfEmpty]! !

!SameGameGameField methodsFor: 'actions' stamp: 'ek 10/30/2007 19:27'!
fillWithStones
	"Add stoneColumns to receiver and fill them with random stones."

	| column |
	maxStoneColumns timesRepeat: [	
			column := SameGameStoneColumn newWithMaxStones: maxStoneRows.
			"maxStoneRows timesRepeat: [ column addMorph: (stoneCatalog atRandom) shallowCopy ]."
			"OLPC fix."
			maxStoneRows timesRepeat: [ column addMorph: (stoneCatalog asOrderedCollection atRandom) shallowCopy ].
			self addMorph: column ]! !

!SameGameGameField methodsFor: 'actions' stamp: 'jfs 7/19/2007 11:53'!
refresh
	"Refresh the game field after a stone group was destroyed."

	self 
		destroyEmptyStoneColumns;
		findStoneGroups;
		countStones.
	
	self hasSubmorphs ifFalse: [score := score + 2000].	"bonus if user managed to clear the game field"
		
	self game updateDisplays
		! !

!SameGameGameField methodsFor: 'actions' stamp: 'ek 6/30/2007 15:45'!
reset

	score := 0.
	
	self 
		removeAllMorphs;
		fillWithStones;
		findStoneGroups;
		countStones! !

!SameGameGameField methodsFor: 'actions' stamp: 'ek 10/30/2007 19:28'!
sizeDownBy: anInteger
	"Size down the game field by the given number of stone rows and stone columns."

	(maxStoneRows - anInteger > minFieldSizeLimit) & (maxStoneColumns - anInteger > minFieldSizeLimit) 
		ifTrue: [
			maxStoneColumns := maxStoneColumns - anInteger.
			maxStoneRows := maxStoneRows - anInteger].
	self adaptExtend! !

!SameGameGameField methodsFor: 'actions' stamp: 'ek 10/30/2007 19:28'!
sizeUpBy: anInteger
	"Size up the game field by the given number of stone rows and columns."

	(maxStoneRows - anInteger < maxFieldSizeLimit) & (maxStoneColumns - anInteger < maxFieldSizeLimit) 
		ifTrue: [
			maxStoneColumns := maxStoneColumns + anInteger.
			maxStoneRows := maxStoneRows + anInteger.].
	self adaptExtend! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
columnCount

	^ self submorphCount! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
game

	^ game! !

!SameGameGameField methodsFor: 'accessing' stamp: 'jfs 7/19/2007 11:50'!
gameIsOver
	"Returns true if there are no more stone groups."

	self stoneColumns do: [ :aColumn | 
		aColumn stones do: [ :aStone | 
			(aStone stoneGroup) size = 1 ifFalse: [^ false] ] ].
	^ true! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
game: aSameGame

	game := aSameGame! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 12:33'!
initializeNumberOfStones

	numberOfStones := Dictionary new
		at: #green put: 0;
	 	at: #yellow put: 0;
		at: #red put: 0;
		at: #blue put: 0;
	 	at: #purple put: 0;
		yourself! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
numberOfStones

	^ numberOfStones! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
stoneCatalog

	^ stoneCatalog! !

!SameGameGameField methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:43'!
stoneColumns

	^ self submorphs! !

!SameGameGameField methodsFor: 'accessing' stamp: 'jfs 7/19/2007 11:57'!
stoneRows
	"Return a collection of integers representing the rows of stones at the game field."

	^ 1 to: maxStoneRows! !

!SameGameGameField methodsFor: 'stone groups' stamp: 'jfs 7/17/2007 12:35'!
findStoneGroups
	"Find groups of stones of same color which are neighboring."	
		
	self stoneColumns do: [:each | each resetStones ].
		
	self stoneColumns do: [:each | each findStoneGroupsInColumn ].
		
	self stoneRows do: [:each | self findStoneGroupsInRow: each ]

	   	   ! !

!SameGameGameField methodsFor: 'stone groups' stamp: 'jfs 7/17/2007 12:35'!
findStoneGroupsInRow: aStoneRow
	"Find groups of stones of same type in given row which are neighboring horizontally."

	| currentStone lastStone |
	lastStone := nil. 
     
	self stoneColumns do: [ :each | 
		currentStone := each stoneAt: aStoneRow.
		currentStone notNil 
			ifTrue: [ 
				(currentStone sameAs: lastStone) 
					ifTrue: [ currentStone stoneGroup merge: lastStone stoneGroup ] ].
		lastStone := currentStone	 ]! !

!SameGameGameField methodsFor: 'initialization' stamp: 'rkrk 11/16/2007 15:45'!
initialize

	super initialize.
	
	minFieldSizeLimit := 4.
	maxFieldSizeLimit := 30.
	
	maxStoneColumns := 18.
	maxStoneRows := 12.
	
	self
		makeLayout;
		makeBackground;
		initializeNumberOfStones;
		fillStoneCatalog;
		reset
! !

!SameGameGameField methodsFor: 'initialization' stamp: 'ek 6/30/2007 17:04'!
makeBackground

	self 
		color: (SameGame color: #blue);
		borderColor: (SameGame color: #darkBlue)! !

!SameGameGameField methodsFor: 'initialization' stamp: 'jfs 7/19/2007 11:45'!
makeLayout

	self 
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		listCentering: #bottomRight;
		borderWidth: 1;
		adaptExtend
		! !

!SameGameGameField methodsFor: 'score' stamp: 'ek 6/24/2007 16:15'!
addToScore: anInteger

	score := score + anInteger.
	self refresh! !

!SameGameGameField methodsFor: 'score' stamp: 'ek 6/30/2007 16:04'!
score

	^ score! !

!SameGameGameField methodsFor: 'event handling' stamp: 'ek 6/30/2007 16:05'!
handlesMouseDown: evt

	^ true! !

!SameGameGameField methodsFor: 'event handling' stamp: 'ek 6/30/2007 16:05'!
mouseDown: evt

	nil
! !


!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:44'!
gameField: aGameField

	gameField := aGameField! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:44'!
stoneColor

	^ stoneColor! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:44'!
stoneColor: aSymbol

	stoneColor := aSymbol! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:44'!
stoneGroup

	^ stoneGroup! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:44'!
stoneGroup: aStoneGroup

	stoneGroup := aStoneGroup! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:47'!
stoneImageHover: anImage

	^ stoneImageHover := anImage! !

!SameGameStone methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:47'!
stoneImage: anImage

	stoneImage := anImage! !

!SameGameStone methodsFor: 'testing' stamp: 'ek 6/24/2007 15:47'!
isSameGameStone

	^ true! !

!SameGameStone methodsFor: 'testing' stamp: 'jfs 7/19/2007 12:27'!
sameAs: aStone
	"Use to test equality instead of =. Method = could not be overridden due to a bug in the Morphic implementation concerning submorphs."

	aStone isSameGameStone ifTrue: [ ^ self stoneColor = aStone stoneColor ].
	^ false! !

!SameGameStone methodsFor: 'actions' stamp: 'ek 6/24/2007 15:47'!
highlight

	self image: stoneImageHover! !

!SameGameStone methodsFor: 'actions' stamp: 'ek 6/24/2007 15:47'!
normal

	self image: stoneImage! !

!SameGameStone methodsFor: 'actions' stamp: 'jfs 7/19/2007 12:25'!
reset
	"Reset the stone group of the receiver."

	stoneGroup := SameGameStoneGroup newWithStone: self! !

!SameGameStone methodsFor: 'event handling' stamp: 'ek 6/24/2007 15:47'!
handlesMouseDown: evt

	^ true! !

!SameGameStone methodsFor: 'event handling' stamp: 'ek 6/24/2007 15:47'!
handlesMouseOver: evt

	^ true! !

!SameGameStone methodsFor: 'event handling' stamp: 'ek 7/1/2007 16:44'!
mouseDown: evt

	| score |
	score := stoneGroup evaluateScore.
	
	stoneGroup size > 1 
		ifTrue: [ 
			stoneGroup destroy.
			gameField addToScore: score ]! !

!SameGameStone methodsFor: 'event handling' stamp: 'ek 6/24/2007 16:17'!
mouseEnter: evt

	stoneGroup viewHighlighted! !

!SameGameStone methodsFor: 'event handling' stamp: 'ek 6/24/2007 15:48'!
mouseLeave: evt

	stoneGroup viewNormal
! !

!SameGameStone methodsFor: 'initialization' stamp: 'ek 6/24/2007 16:16'!
initialize

	super initialize.
	
	self reset! !


!SameGameStone class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:40'!
blueStoneAt: aGameField

	^ self new 
		stoneColor: #blue;
		stoneImage: (SameGame image: #blueStone);
		stoneImageHover: (SameGame image: #orangeStone);
		image: (SameGame image: #blueStone);
		gameField: aGameField;
		yourself! !

!SameGameStone class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
greenStoneAt: aGameField

	^ self new
		stoneColor: #green;
		stoneImage: (SameGame image: #greenStone);
		stoneImageHover: (SameGame image: #orangeStone);
		image: (SameGame image: #greenStone);
		gameField: aGameField;
		yourself! !

!SameGameStone class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
purpleStoneAt: aGameField

	^ self new 
		stoneColor: #purple;
		stoneImage: (SameGame image: #purpleStone);
		stoneImageHover: (SameGame image: #orangeStone);
		image: (SameGame image: #purpleStone);
		gameField: aGameField;
		yourself! !

!SameGameStone class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
redStoneAt: aGameField

	^ self new 
		stoneColor: #red;
		stoneImage: (SameGame image: #redStone);
		stoneImageHover: (SameGame image: #orangeStone);
		image: (SameGame image: #redStone);
		gameField: aGameField;
		yourself! !

!SameGameStone class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
yellowStoneAt: aGameField

	^ self new 
		stoneColor: #yellow;
		stoneImage: (SameGame image: #yellowStone);
		stoneImageHover: (SameGame image: #orangeStone);
		image: (SameGame image: #yellowStone);
		gameField: aGameField;
		yourself! !


!SameGameStoneColumn methodsFor: 'actions' stamp: 'ek 7/1/2007 16:45'!
destroyIfEmpty

	self hasSubmorphs ifFalse:[ self delete]! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'jfs 7/17/2007 12:34'!
findStoneGroupsInColumn
	"Find groups of stones of same type in this stone column which are neighboring vertically."

	| lastStone |
	lastStone := nil.
	      
	self stones do: [ :each | 
		(each sameAs: lastStone) ifTrue: [
			 each stoneGroup merge: lastStone stoneGroup ].
		lastStone := each ]! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'jfs 7/17/2007 12:06'!
maxStones: anInteger

	maxStones := anInteger! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'ek 7/19/2007 17:20'!
resetStones

	^ self stones do: [ :each | each reset ]! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'jfs 7/19/2007 12:33'!
stoneAt: aRow
	"Returns stone in given row number (starting with 1 for the topmost row). Returns nil if no stone is placed in this row."
	
	| submorphIndex |
	submorphIndex := aRow - maxStones + self stoneCount.

	self submorphs do: [ :each | 
		(self submorphIndexOf: each) = submorphIndex ifTrue: [ ^ each ]].
	
	^ nil! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:48'!
stoneCount

	^ self submorphCount! !

!SameGameStoneColumn methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:48'!
stones

	^ self submorphs! !

!SameGameStoneColumn methodsFor: 'initialization' stamp: 'ek 6/24/2007 15:48'!
initialize

	super initialize.
	
	maxStones := 0! !

!SameGameStoneColumn methodsFor: 'initialization' stamp: 'ek 6/30/2007 17:17'!
makeLayout

	| stoneSize |
	stoneSize := 25.

	self	
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		listCentering: #bottomRight;
		hResizing: #shrinkWrap;
		extent: stoneSize @ stoneSize*maxStones;
		color: Color transparent! !


!SameGameStoneColumn class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
newWithMaxStones: anInteger

	^ self new
		maxStones: anInteger;
		makeLayout;
		yourself! !


!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:50'!
gameField

	^ gameField! !

!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:50'!
gameField: aGameField

	gameField := aGameField! !

!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:50'!
stoneColor

	^ stoneColor! !

!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:50'!
stoneColor: aSymbol

	stoneColor := aSymbol! !

!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:50'!
string

	^ string! !

!SameGameStoneDisplay methodsFor: 'accessing' stamp: 'ek 6/24/2007 15:51'!
value: anInteger

	string contents: anInteger asString! !

!SameGameStoneDisplay methodsFor: 'initialization' stamp: 'ek 6/24/2007 15:52'!
initialize

	super initialize.
	
	value := 0.
	
	self
		setLabel;
		makeLayout! !

!SameGameStoneDisplay methodsFor: 'initialization' stamp: 'ek 6/30/2007 16:09'!
makeLayout

	self
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		cellInset: 5;
		width: 75;
		height: 25;
		borderWidth: 0;
		color: Color transparent! !

!SameGameStoneDisplay methodsFor: 'initialization' stamp: 'ek 6/30/2007 17:02'!
setLabel

	self addMorph:
		(string := StringMorph new
			contents: '0';
			fontName: 'ComicSansMS' size: 30;
			color:  (SameGame color: #darkBlue);
			yourself)! !

!SameGameStoneDisplay methodsFor: 'initialization' stamp: 'jfs 7/19/2007 12:36'!
setStone
	"Add stone morph associated to stoneColor to receiver."

	self addMorph: 
		(ImageMorph new 
			image: (gameField stoneCatalog at: stoneColor) image)! !


!SameGameStoneDisplay class methodsFor: 'instance creation' stamp: 'ek 6/24/2007 15:41'!
newWithStoneColor: aSymbol at: aGameField

	^ self new
		gameField: aGameField;
		stoneColor: aSymbol;
		setStone;
		yourself! !


!SameGameStoneGroup methodsFor: 'initializing' stamp: 'ek 6/24/2007 15:49'!
initialize

	super initialize
! !

!SameGameStoneGroup methodsFor: 'actions' stamp: 'ek 6/24/2007 16:00'!
destroy

	self do: [ :each | each delete]! !

!SameGameStoneGroup methodsFor: 'actions' stamp: 'jfs 7/19/2007 12:39'!
merge: aStoneGroup
	"Add all stones of given stone group to receiver."

	aStoneGroup do: [ :each | each stoneGroup: self ].
	self addAll: aStoneGroup
	
	  ! !

!SameGameStoneGroup methodsFor: 'actions' stamp: 'ek 6/24/2007 15:50'!
removeStones

	self do: [ :aStone | self remove: aStone]! !

!SameGameStoneGroup methodsFor: 'actions' stamp: 'ek 6/24/2007 15:50'!
viewHighlighted

	self size > 1 ifTrue: [ self do: [ :each | each highlight] ]! !

!SameGameStoneGroup methodsFor: 'actions' stamp: 'ek 6/24/2007 15:50'!
viewNormal

	self do: [ :each | each normal]! !

!SameGameStoneGroup methodsFor: 'accessing' stamp: 'jfs 7/19/2007 12:38'!
evaluateScore
	"Evaluate score the user earns when eliminating the receiver."

	^ (self size + 1) squared! !


!SameGameStoneGroup class methodsFor: 'instance creation' stamp: 'jfs 6/4/2007 19:22'!
newWithStone: aStone
	"Creates new StoneGroup instance which contains aStone"
	
	^ SameGameStoneGroup new
		add: aStone;
		yourself! !

SameGame initialize!
SameGame class removeSelector: #selectorsForResources!
SameGame class
	instanceVariableNames: 'description'!

!SameGame class reorganize!
('parts bin' descriptionForPartsBin)
('default settings' addImages defaultDirectory defaultExtent images setColors)
('instance creation' startNewGame)
('accessing' color: description image:)
('installation' formFromFileNamed: install)
('initialization' initialize)
!

BorderedMorph subclass: #SameGame
	instanceVariableNames: 'controlMenuMorph fieldMorph stoneDisplays scoreDisplayMorph'
	classVariableNames: 'Colors Images'
	poolDictionaries: ''
	category: 'SameGame-Game'!
"Postscript:
Leave the line above, and replace the rest of this comment by a useful one.
Executable statements should follow this comment, and should
be separated by periods, with no exclamation points (!!).
Be sure to put any further comments in double-quotes, like this one."
!

